#<profile4xi profile = "10" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "rebuild_socket4xi.py">
#<abstract4xi abstract = "providing function rebuild_socket4xi"/>
#<refer4xi reference = "reduction.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'rebuild_socket4xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
    import socket
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import socket
    import traceback         
except:
    pass
    
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:
    from multiprocessing.reduction import rebuild_socket
except:
    print("rebuild_socket4xi.py: *** failed in from multiprocessing.reduction import rebuild_socket ***")
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from _get_listener4xi import _get_listener4xi

from close4xi import close4xi
from fromfd4xi import fromfd4xi
from recv_handle4xi import recv_handle4xi
from rebuild_connection4xi import rebuild_connection4xi
from rebuild_handle4xi import rebuild_handle4xi

from print4xi import print4xi
from time4xi import time4xi
#"
#/>

 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "rebuild_socket4xi" usage = "
#   reduction.py: 
#       reduce_socket(s):
#           df = DupFd(s.fileno())
#           return rebuild_socket, (df, s.family, s.type, s.proto)
#   reduce_socket4xi.py: 
#       reduce_socket4xi(s):
#           df = DupFd4Xi(s.fileno())
#           return rebuild_socket4xi, (df, s.family, s.type, s.proto)
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
if sys.platform == 'win32': 
    def rebuild_socket4xi(ds):
        '''
        returns rebuild_socket4xi, (DupSocket4Xi(s),)
        '''
        class2print = "rebuild_socket4xi"
        method2print = "rebuild_socket4xi"
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2) - Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 1/2a) started" + "; rebuild-socket4xi-started: " + str(rebuild_socket4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        message2print = "{(Step 1/2b) passed" + "; ds-passed-to-rebuild-socket4xi: " + str(ds) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
            
        
        detach2xi = ds.detach()
        
        message2print = "{(Step 1/2c) set" + "; detach2xi-set-by-rebuild-socket4xi to: " + str(detach2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
                 
               
        # -----------------------------------------------------------------------------------------    
        # (Step 2/2) - Windows
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 2/2) returning" + "; detach2xi-returning-by-rebuild-socket4xi: " + str(detach2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
    
        return detach2xi
else:
    def rebuild_socket(df, family, type, proto):
        class2print = "rebuild_socket4xi"
        method2print = "rebuild_socket4xi"
    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3) - Non Windows - parameters differ
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 1/3a) started" + "; rebuild-socket4xi-started: " + str(rebuild_socket4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        message2print = "{(Step 1/3b) passed" + "; df-passed-to-rebuild-socket4xi: " + str(df) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 1/3c) passed" + "; family-passed-to-rebuild-socket4xi: " + str(family) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        message2print = "{(Step 1/3d) passed" + "; type-passed-to-rebuild-socket4xi: " + str(type) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        message2print = "{(Step 1/3e) passed" + "; proto-passed-to-rebuild-socket4xi: " + str(proto) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
    
        # -----------------------------------------------------------------------------------------    
        # (Step 2/3) - Non Windows
        # -----------------------------------------------------------------------------------------         
        fd2xi = df.detach()
        message2print = "{(Step 2/3a) got" + "; fd2xi-got-by-rebuild-socket4xi to: " + str(fd2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        s2xi = socket.fromfd(fd2xi, family, type, proto)
        message2print = "{(Step 2/3b) got" + "; s2xi-got-by-rebuild-socket4xi to: " + str(s2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 2/3c) passing" + "; fd2xi-to-os-close-by-socket4xi: " + str(fd2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
    
        # -----------------------------------------------------------------------------------------    
        # (Step 3/3) - Non Windows
        # -----------------------------------------------------------------------------------------          
        message2print = "{(Step 3/3b) passing" + "; fd2xi-passing-to-os-close-by-rebuild-socket4xi to: " + str(s2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        os.close(fd2xi)
        
        message2print = "{(Step 3/3b) returning" + "; s2xi-returning-by-rebuild-socket4xi to: " + str(s2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        
        return s2xi              
        
#</function4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of rebuild_socket4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>
